from __future__ import annotations

from typing import Union, Optional

from multimethod import multimethod
from pydantic import StrictInt, StrictBool, validate_arguments
from titlecase import titlecase

try:
    import discord
except ImportError as err:
    err.msg = "alianator couldn't find Pycord in your environment. Install it, then try again."
    raise err


class _PermissionFlagMeta(type):
    def __new__(mcs, name, bases, namespace, **kwargs):
        name = namespace.get("__name__", "discord.Permissions.VALID_FLAGS")
        return super().__new__(mcs, name, bases, namespace, **kwargs)

    def __instancecheck__(self, instance):
        return instance in discord.Permissions.VALID_FLAGS


class _PermissionFlag(metaclass=_PermissionFlagMeta):
    pass


@validate_arguments(config=dict(arbitrary_types_allowed=True))
def resolve(
    arg: Union[
        discord.Permissions,
        StrictInt,
        _PermissionFlag,
        tuple[_PermissionFlag, StrictBool],
        list[_PermissionFlag],
        list[tuple[_PermissionFlag, StrictBool]],
    ],
    mode: Optional[StrictBool] = True,
) -> list[str]:
    """
    Resolves Discord permission names from their API flags to their user-facing aliases.

    Parameters
    ----------
    arg : Union[discord.Permissions, int, str, tuple[str, bool], list[str], list[tuple[str, bool]]]
        An object representing the permissions to resolve.
    mode : bool, optional
        A boolean flag that determines which permissions will be resolved. If True, only granted permissions will be
        resolved. If False, only denied permissions will be resolved. If None, all permissions will be resolved.
        Defaults to True. If the function recieves a string or list of strings, all permissions will be resolved
        regardless of the value of this flag.

    Returns
    -------
    list[str]
        A list of resolved permissions.
    """

    def resolver(names: list[str]) -> list[str]:
        # manually define resolutions that can't be accomplished by simple character substitution
        resolutions = {
            "send_messages": "Send Messages and Create Posts",
            "send_messages_in_threads": "Send Messages in Threads and Posts",
            "external_emojis": "Use External Emoji",
            "external_stickers": "Use External Stickers",
            "manage_emojis": "Manage Emojis and Stickers",
            "manage_threads": "Manage Threads and Posts",
            "mention_everyone": "Mention \\@everyone, \\@here, and All Roles",
            "moderate_members": "Timeout Members",
            "send_tts_messages": "Send Text-to-Speech Messages",
            "start_embedded_activities": "Use Activities",
            "stream": "Video",
            "use_slash_commands": "Use Application Commands",
            "use_voice_activation": "Use Voice Activity",
        }

        return [
            resolutions.get(name)
            or titlecase(name.replace("_", " ").replace("guild", "server"))
            for name in names
        ]

    @multimethod
    def _resolve(permissions: Union[discord.Permissions, list[tuple[str, bool]]]):
        return resolver(
            [perm for perm, status in permissions if mode in [status, None]]
        )

    @multimethod
    def _resolve(permissions: list[str]):
        return resolver(permissions)

    @multimethod
    def _resolve(permissions: Union[str, tuple[str, bool]]):
        return _resolve([permissions])

    @multimethod
    def _resolve(permissions: int):
        return _resolve(discord.Permissions(permissions))

    return _resolve(arg)



import platform
from pathlib import Path

import discord
from jinja2 import Environment, FileSystemLoader

import alianator

here = Path(__file__).parent

uname = platform.uname()

data = {
    "alianator_version": alianator.__version__,
    "pycord_version": discord.__version__,
    "python_implementation": platform.python_implementation(),
    "python_version": platform.python_version(),
    "system_info": {
        "name": uname.system,
        "version": uname.release,
        "architecture": uname.machine,
    },
}

env = Environment(loader=FileSystemLoader(here / "templates"))

print(env.get_template("diagnostic.yml.jinja").render(data))



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['alianator']

package_data = \
{'': ['*'], 'alianator': ['templates/*']}

install_requires = \
['jinja2>=3.1.2,<4.0.0',
 'multimethod>=1.9.1,<2.0.0',
 'pydantic>=1.10.4,<2.0.0',
 'titlecase>=2.3,<3.0']

setup_kwargs = {
    'name': 'alianator',
    'version': '3.2.1',
    'description': 'A Discord permission name resolver for Pycord',
    'long_description': '# alianator\n\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/alianator?logo=python&logoColor=white&style=for-the-badge)](https://pypi.org/project/alianator)\n[![PyPI](https://img.shields.io/pypi/v/alianator?logo=pypi&color=green&logoColor=white&style=for-the-badge)](https://pypi.org/project/alianator)\n[![GitHub release (latest SemVer)](https://img.shields.io/github/v/release/celsiusnarhwal/alianator?logo=github&color=orange&logoColor=white&style=for-the-badge)](https://github.com/celsiusnarhwal/alianator/releases)\n[![PyPI - License](https://img.shields.io/pypi/l/alianator?color=03cb98&style=for-the-badge)](https://github.com/celsiusnarhwal/alianator/blob/master/LICENSE)\n\nalianator is a Discord permission name resolver for [Pycord](https://github.com/Pycord-Development/pycord). \nIt takes Discord\'s API permission flags (e.g. `manage_guild`) and resolves them to their user-facing aliases (e.g. `Manage Server`).\n\n## Installation\n\n```bash\n$ pip install alianator\n```\n\n[Pycord](https://github.com/Pycord-Development/pycord) is not included as a dependency of alianator, but nonetheless must \nbe installed for it to work. If alianator is unable to import the `discord` namespace, it will raise an `ImportError`.\n\n## Usage\n\nalianator can resolve aliases from `discord.Permissions` objects, integers, strings, tuples, lists of strings, and lists\nof tuples.\n\n```python\nimport alianator\n\nalianator.resolve(arg, mode=mode)\n```\n\nThe optional `mode` flag can be used to specify which permissions should be resolved. If `mode` is `True`, only granted\npermissions will be resolved; if `mode` is `False`, only denied permissions will be resolved; if `mode` is `None`, all\npermissions will be resolved. If `mode` is not explicitly specified, it will default to `True`.\n\n```python\nimport alianator\nimport discord\n\n# Resolving from a discord.Permissions object\nperms = discord.Permissions.general()\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Manage Channels\', \'Manage Server\', \'View Audit Log\', \'Read Messages\', \'View Server Insights\', \'Manage Roles\', \'Manage Webhooks\', \'Manage Emojis and Stickers\']\n\n\n# Resolving from an integer\nperms = 3072\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'View Channel\', \'Send Messages and Create Posts\']\n\n\n# Resolving from a string\nperms = "send_tts_messages"\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Send Text-To-Speech Messages\']\n\n\n# Resolving from a tuple\nperms = ("moderate_members", True)\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Timeout Members\']\n\n\n# Resolving from a list of strings\nperms = ["manage_guild", "manage_emojis"]\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Manage Server\', \'Manage Emojis and Stickers\']\n\n\n# Resolving from a list of tuples\nperms = [("use_slash_commands", True), ("use_voice_activation", True)]\naliases = alianator.resolve(perms)\nprint(aliases)\n# [\'Use Application Commands\', \'Use Voice Activity\']\n```\n\nThat\'s about all there is to it. alianator does one thing and does it well.\n\n## License\n\nalianator is released under the [MIT License](https://github.com/celsiusnarhwal/alianator/blob/master/LICENSE.md).\n',
    'author': 'celsius narhwal',
    'author_email': 'celsiusnarhwal@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/celsiusnarhwal/alianator',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
